@@ -15,31 +15,47 @@
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 \*************************************************************************/
 
 //--- LOCAL ---
+#include "configdialog/dlgtimezone.h"
 #include "weatherdataprocessor.h"
 #include "utils.h"
 #include "logger/streamlogger.h"
+#include "math.h"
+#include "float.h"
 
 //--- QT4 ---
 #include <QDir>
 #include <QFile>
+#include <QFontMetrics>
 #include <QHash>
+#include <QTextLayout>
 
 //--- KDE4 ---
+#include <KDateTime>
+#include <KGlobalSettings>
+#include <KTimeZones>
+#include <KSystemTimeZones>
 
-#include <values.h>
 #if KDE_IS_VERSION(4,3,70)
 	#include <KUnitConversion/Value>
 #endif
 
+#include <QMessageBox>
+
+#define MAX_PROPERTY_TEXT_WIDTH 258.0f
+#include <sys/stat.h>
+
 const QString WeatherDataProcessor::CacheDirectory = QDir::homePath() + QLatin1String("/.cache/yawp");
+static const char vWeekdays [7][4] = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};
 
 
 struct WeatherDataProcessor::Private
 {
+	const Yawp::Storage           * pStorage;
+
 	QHash<QString, QString>         vConditionIcons;
 
 	YAWP_DISTANCE_UNIT              distanceSystem;
@@ -47,6 +63,8 @@ struct WeatherDataProcessor::Private
 	YAWP_SPEED_UNIT                 speedSystem;
 	YAWP_TEMPERATURE_UNIT           temperatureSystem;
 
+	bool                            requestTimeZone;
+
 	QHash<Yawp::DetailsProperty, short> vDetailsPropertyMap;
 
 	bool initIconMap(const QString & resource);
@@ -76,12 +94,25 @@ struct WeatherDataProcessor::Private
 	bool findDateForWeekday( QDate & currDate, const QString & sLookingDayName ) const;
 
 	QString getSourceCacheFileName( const CityWeather & pCity ) const;
+
+	QString createPropertyText( const Yawp::DetailsProperty, const YawpDay & pDay, const YawpWeather & pWeather ) const;
+	void createPropertyStringList( const YawpDay & day, YawpWeather & weather ) const;
+	QSizeF doTextLayout( QTextLayout & textLayout, qreal dMaxWidth, qreal dOffset ) const;
+
+	inline void
+	addPropertyItem( QMap<short, Yawp::DetailsProperty> & vMap, Yawp::DetailsProperty property ) const
+	{
+		if( vDetailsPropertyMap.contains(property) )
+			vMap[ vDetailsPropertyMap.value(property) ] = property;
+	}
 };
 
-WeatherDataProcessor::WeatherDataProcessor()
+WeatherDataProcessor::WeatherDataProcessor( const Yawp::Storage * pStorage )
 	: d( new WeatherDataProcessor::Private )
 {
+	d->pStorage = pStorage;
 	d->initIconMap( ":/iconnames.conf" );
+	d->requestTimeZone = false;
 
 	QDir dir(CacheDirectory);
 	if( !dir.exists() )
@@ -105,6 +136,9 @@ void WeatherDataProcessor::setTemperatureSystem( YAWP_TEMPERATURE_UNIT unitsyste
 YAWP_SPEED_UNIT WeatherDataProcessor::speedSystem() const {return d->speedSystem;}
 void WeatherDataProcessor::setSpeedSystem( YAWP_SPEED_UNIT unitsystem ) {d->speedSystem = unitsystem;}
 
+void WeatherDataProcessor::setRequestTimeZone(bool request) {d->requestTimeZone = request;}
+bool WeatherDataProcessor::requestTimeZone() const {return d->requestTimeZone;}
+
 bool
 WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
                                       const Plasma::DataEngine::Data & data ) const
@@ -116,17 +150,8 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 	if( data.isEmpty() || data.contains("validate") )
 		return false;
 
-	cityInfo.deleteAllDays();
-
 /*** FOR DEBUG - PRINTS A SORTED LIST OF ALL KEYS THAT THIS PROVIDER/ION SUPPORTS ***/
-	dDebug() << cityInfo.localizedCityString() << cityInfo.provider();
-	QStringList sKeyList;
-	QHash<QString, QVariant>::const_iterator itKeys;
-	for( itKeys = data.begin(); itKeys != data.end(); ++itKeys )
-		sKeyList.append( itKeys.key() );
-	sKeyList.sort();
-	foreach( QString sKey, sKeyList )
-		qDebug() << "     " << sKey << "\t\t -> " << data.value(sKey);
+	dInfo() << cityInfo.localizedCityString() << cityInfo.provider() << data;
 /************************************************************************************/
 
 	YAWP_TEMPERATURE_UNIT fromTempSystem
@@ -140,7 +165,22 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 	YAWP_PRESSURE_UNIT    fromPressureSystem
 		= (YAWP_PRESSURE_UNIT)data.value("Pressure Unit").toInt();
 
-//--- Get weather forecast information ---
+	//--- delete all old days ---
+	cityInfo.deleteAllDays();
+
+	QString sObservation( data.value("Observation Period").toString() );
+	if( !sObservation.isEmpty() && sObservation.compare("N/A") != 0 )
+	{
+		QDateTime obsTime = QDateTime::fromString( sObservation, "dd.MM.yyyy @ h:mm");
+		if( !obsTime.isValid() )
+			obsTime = KDateTime::fromString(sObservation, "%d.%m.%Y @ %H:%M %z")/*.toLocalZone()*/.dateTime();
+
+		cityInfo.setObservationPeriode(obsTime);
+	}
+	else
+		cityInfo.setObservationPeriode( QDateTime() );
+
+	//--- Get weather forecast information ---
 	cityInfo.setCredit( data.value("Credit").toString() );
 	cityInfo.setCreditURL( data.value("Credit Url").toString() );
 	int iForecastDays = data.value("Total Weather Days", "0").toInt();
@@ -158,17 +198,25 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		YawpWeather * pWeather = NULL;
 
 		//--- find the date to the dayname ---
-		if( vTokens.at(0).compare("Today", Qt::CaseInsensitive) == 0 ||
-		    vTokens.at(0).compare( i18n("Today"), Qt::CaseInsensitive) == 0 )
+		if( vTokens.at(0).compare( "Today", Qt::CaseInsensitive) == 0 ||
+		    vTokens.at(0).compare( i18n("Today"), Qt::CaseInsensitive) == 0 ||
+		    vTokens.at(0).startsWith( "day", Qt::CaseInsensitive) ||  // <--- envcan specific
+		    vTokens.at(0).startsWith( i18n("day"), Qt::CaseInsensitive) )	// <--- wettercom specific
 		{
+			dTracing() << "Adding Today for " << vTokens;
 			iTodayDays += 1;
 			pLastDay = new YawpDay;
 			cityInfo.days().append( pLastDay );
 			pWeather = &pLastDay->weather();
 		}
 		else if( vTokens.at(0).compare("Tonight", Qt::CaseInsensitive) == 0 ||
-		    vTokens.at(0).compare( i18n("Tonight"), Qt::CaseInsensitive) == 0 )
+		    vTokens.at(0).compare( i18n("Tonight"), Qt::CaseInsensitive) == 0 ||
+		    vTokens.at(0).startsWith( "Nite", Qt::CaseInsensitive) ||        // <--- envcan specific
+		    vTokens.at(0).startsWith( i18n("Nite"), Qt::CaseInsensitive) ||  // <--- envcan specific
+		    vTokens.at(0).startsWith( "Night", Qt::CaseInsensitive) ||       // <--- wettercom specific
+		    vTokens.at(0).startsWith( i18n("Night"), Qt::CaseInsensitive) )  // <--- wettercom specific
 		{
+			dTracing() << "Adding Tonight for " << vTokens;
 			iTodayDays += 1;
 			if( !pLastDay )
 			{
@@ -185,12 +233,15 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		}
 		else if( !currDate.isValid() )
 		{
+			dTracing() << "Date is not falid -> check " << vTokens;
 			QString sShortDay( vTokens.at(0) );
 			bool bNightTime = d->isNightTime( sShortDay );
 
-			QDate date( QDate::currentDate().addDays( iTodayDays > 0 ? 2 : 1 ) );
+			QDate date( cityInfo.localTime().date().addDays( iTodayDays > 0 ? 2 : 1 ) );
+
 			if( d->findDateForWeekday( date, sShortDay ) )
 			{
+				dTracing() << "Adding weather for" << date;
 				int iFirstDateIndexDay = cityInfo.days().count();
 				sLastDayName = sShortDay;
 				currDate = date;
@@ -212,6 +263,7 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		}
 		else
 		{
+			dTracing() << "check -> " << vTokens;
 			QString sShortDay( vTokens.at(0) );
 			bool bNightTime = d->isNightTime( sShortDay );
 			if( bNightTime && pLastDay && sLastDayName.compare( sShortDay ) == 0 )
@@ -259,18 +311,34 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 	else
 		pDay = cityInfo.days().at(0);
 
-	pDay->setSunrise( QTime::fromString(data.value("Sunrise At").toString(), "h:m") );
-	pDay->setSunset(  QTime::fromString(data.value("Sunset At").toString(), "h:m") );
+	/*** PARSE SUNRISE AND SUNSET ***/
+	KDateTime sunTime;
+	sunTime = KDateTime::fromString(data.value("Sunrise At").toString(), "%A %B %e, %Y at %H:%M %Z", KSystemTimeZones::timeZones(), true);
+	if( sunTime.isValid() )
+		pDay->setSunrise( sunTime.time() );
+	else
+		pDay->setSunrise( QTime::fromString(data.value("Sunrise At").toString(), "h:m") );
+
+	sunTime = KDateTime::fromString(data.value("Sunset At").toString(), "%A %B %e, %Y at %H:%M %Z", KSystemTimeZones::timeZones(), true);
+	if( sunTime.isValid() )
+		pDay->setSunset( sunTime.time() );
+	else
+		pDay->setSunset(  QTime::fromString(data.value("Sunset At").toString(), "h:m") );
+
 
 	YawpWeather & weather
-		= (pDay->hasNightValues() && !Utils::IsDayTime(pDay->sunrise(), pDay->sunset()) ?
-			pDay->nightWeather() : pDay->weather());
+		= (pDay->hasNightValues() && !cityInfo.isDayTime(pDay) ? pDay->nightWeather() : pDay->weather());
 
-	weather.setCurrentIconName( mapConditionIcon( data.value("Condition Icon").toString() ) );
-	weather.setCurrentTemperature( d->convertTemp(  data.value("Temperature").toString(), fromTempSystem ) );
-	weather.setWindSpeed( d->convertSpeed( data.value("Wind Speed").toString(),  fromSpeedSystem ) );
-	weather.setDewpoint( d->convertTemp( data.value("Dewpoint").toString(), fromDewTempSystem ) );
-	weather.setVisibility( d->convertDistance( data.value("Visibility").toString(), fromVisibilitySystem ) );
+	if( data.contains("Condition Icon") )
+		weather.setCurrentIconName( mapConditionIcon( data.value("Condition Icon").toString() ) );
+	if( data.contains("Temperature") )
+		weather.setCurrentTemperature( d->convertTemp(  data.value("Temperature").toString(), fromTempSystem ) );
+	if( data.contains("Wind Speed") )
+		weather.setWindSpeed( d->convertSpeed( data.value("Wind Speed").toString(),  fromSpeedSystem ) );
+	if( data.contains("Dewpoint") )
+		weather.setDewpoint( d->convertTemp( data.value("Dewpoint").toString(), fromDewTempSystem ) );
+	if( data.contains("Visibility") )
+		weather.setVisibility( d->convertDistance( data.value("Visibility").toString(), fromVisibilitySystem ) );
 
 	weather.setPressure( d->convertPressure( data.value("Pressure").toString(), fromPressureSystem ) );
 	sTmp = data.value("Pressure Tendency").toString();
@@ -293,8 +361,9 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 
 	//--- remove all old days ---
 	//
+	dTracing() << "Check and remove old days";
 	bool bRemovedOldDays( false );
-	currDate = QDate::currentDate();
+	currDate = cityInfo.localTime().date();
 	QDate yesterday = currDate.addDays(-1);
 	while( cityInfo.days().count() > 0 )
 	{
@@ -310,6 +379,7 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		}
 		else
 		{
+			dTracing() << "Remove day" << pDay->date() << "- locale today is" << currDate;
 			bRemovedOldDays = true;
 			delete cityInfo.days().takeFirst();
 		}
@@ -321,15 +391,10 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 	{
 		YawpDay * pDay = (*itDay);
 
-		if( (*itDay)->sunrise().isValid() || (*itDay)->sunset().isValid() )
-		{
-			pDay->weather().addProperty( Yawp::SunriseSunset );
-			pDay->nightWeather().addProperty( Yawp::SunriseSunset );
-		}
-		sortPropertyKeys( pDay->weather() );
+		d->createPropertyStringList( *pDay, pDay->weather() );
 
 		if( pDay->hasNightValues() )
-			sortPropertyKeys( pDay->nightWeather() );
+			d->createPropertyStringList( *pDay, pDay->nightWeather() );
 	}
 
 	if( !bRemovedOldDays )
@@ -348,6 +413,9 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 bool
 WeatherDataProcessor::updateCountryInfo( CityWeather & cityInfo, const Plasma::DataEngine::Data & data ) const
 {
+	if( data.isEmpty() || data.contains("validate") )
+		return false;
+
 	/*   Some weatherengines provides more informations for a certain location
 	 *   doing a data request than searching for a city, so we update the locationinformations here.
 	 */
@@ -356,7 +424,7 @@ WeatherDataProcessor::updateCountryInfo( CityWeather & cityInfo, const Plasma::D
 	sCountry = data.value("Country").toString();
 	if( !sCountry.isEmpty() &&
 	    cityInfo.country().compare( sCountry ) != 0 &&
-        Utils::GetCountryCode( sCountry, sCountryCode ) )
+        Utils::GetCountryCode( sCountry, sCountryCode, d->pStorage ) )
 	{
 		cityInfo.setCountry( sCountry );
 		cityInfo.setCountryCode( sCountryCode );
@@ -374,7 +442,7 @@ WeatherDataProcessor::updateCountryInfo( CityWeather & cityInfo, const Plasma::D
 				cityInfo.setCountry( sCountry );
 				bUpdate = true;
 			}
-			if( cityInfo.countryCode().isEmpty() && Utils::GetCountryCode( sCountry, sCountryCode ) )
+			if( cityInfo.countryCode().isEmpty() && Utils::GetCountryCode( sCountry, sCountryCode, d->pStorage ) )
 			{
 				cityInfo.setCountryCode( sCountryCode );
 				bUpdate = true;
@@ -450,33 +518,6 @@ WeatherDataProcessor::createDetailsPropertyMap( const QList<Yawp::DetailsPropert
 	}
 }
 
-bool
-WeatherDataProcessor::sortPropertyKeys( YawpWeather & weather ) const
-{
-	if( d->vDetailsPropertyMap.isEmpty() )
-		return false;
-	if( weather.propertyKeys().count() <= 1 )	// nothing to sort
-		return true;
-
-	QMap<short, Yawp::DetailsProperty> vMap;
-	QList<Yawp::DetailsProperty>::const_iterator itList = weather.propertyKeys().begin();
-	for( ; itList != weather.propertyKeys().end(); ++itList )
-	{
-		if( d->vDetailsPropertyMap.contains( *itList ) )
-		{
-			short iIndex = d->vDetailsPropertyMap.value( *itList );
-			vMap[ iIndex ] = *itList;
-		}
-	}
-
-	weather.propertyKeys().clear();
-	QMap<short, Yawp::DetailsProperty>::const_iterator itMap = vMap.begin();
-	for( ; itMap != vMap.end(); ++itMap )
-		weather.addProperty( itMap.value() );
-	return true;
-}
-
-
 bool
 WeatherDataProcessor::setForecastValues(
 		YawpWeather                   & weather,
@@ -493,7 +534,6 @@ WeatherDataProcessor::setForecastValues(
 		weather.setDescription( vForecast.at(2) );
 		weather.setHighTemperature( d->convertTemp(vForecast.at(3), fromTempSystem) );
 		weather.setLowTemperature(  d->convertTemp(vForecast.at(4), fromTempSystem) );
-
 		bReturn = true;
 	}
 	return bReturn;
@@ -603,10 +643,10 @@ WeatherDataProcessor::Private::convertDistance( const QString & value,
 
 	if( fromSystem != distanceSystem )
 	{
-#if KDE_IS_VERSION(4,3,70)
+#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
 		KUnitConversion::Value v( (double) fValue, fromSystem);
 		fValue = v.convertTo(distanceSystem).number();
-#elif KDE_VERSION_MINOR >= 3
+#elif KDE_VERSION_MINOR == 3
 		fValue = WeatherUtils::convertDistance( fValue, fromSystem, distanceSystem );
 #else
 		fValue = WeatherUtils::convert( fValue, fromSystem, distanceSystem );
@@ -624,7 +664,7 @@ WeatherDataProcessor::Private::convertPressure( const QString & value,
 		return SHRT_MAX;
 	if( fromSystem != pressureSystem )
 	{
-#if KDE_IS_VERSION(4,3,70)
+#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
 		KUnitConversion::Value v( (double) fValue, fromSystem);
 		fValue = v.convertTo(pressureSystem).number();
 #elif KDE_VERSION_MINOR >= 3
@@ -647,7 +687,7 @@ WeatherDataProcessor::Private::convertTemp( const QString & value,
 
 	if( fromSystem != temperatureSystem )
 	{
-#if KDE_IS_VERSION(4,3,70)
+#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
 		KUnitConversion::Value v( (double) fValue, fromSystem);
 		fValue = v.convertTo(temperatureSystem).number();
 #elif KDE_VERSION_MINOR >= 3
@@ -669,7 +709,7 @@ WeatherDataProcessor::Private::convertSpeed( const QString & value,
 
 	if( fromSystem != speedSystem )
 	{
-#if KDE_IS_VERSION(4,3,70)
+#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
 		KUnitConversion::Value v( (double) fValue, fromSystem);
 		fValue = v.convertTo(speedSystem).number();
 #elif KDE_VERSION_MINOR >= 3
@@ -733,16 +773,40 @@ WeatherDataProcessor::Private::isNightTime( QString & sShortDay ) const
 bool
 WeatherDataProcessor::Private::findDateForWeekday( QDate & currDate, const QString & sLookingDayName ) const
 {
+	QString sLookingDayNameUtf8 = sLookingDayName.toUtf8();
+	QString vWeekdayUtf8;
+	KDateTime kDate(currDate);
+
 	for(int iOffset = 0; iOffset < 6; ++iOffset )
 	{
-		QString sCurrDateName( currDate.toString("ddd") );
+		dTracing() << "dayoffset" << iOffset << " -> " << currDate << "(dayindex" << currDate.dayOfWeek() << ")";
+		int iWeekday = currDate.dayOfWeek() -1;
+		if( iWeekday < 0 || iWeekday >= 7 )
+			return false;	// this should not happen
 
-		if( sLookingDayName.contains( sCurrDateName ) ||
-		    sLookingDayName.contains( i18n(sCurrDateName.toUtf8().constData()) ) )
-		{
+		dTracing() << "  1) compare" << sLookingDayNameUtf8 << " with " << iWeekday << " -> " << QString(vWeekdays[iWeekday]).toUtf8();
+		if( sLookingDayNameUtf8.startsWith( QString(vWeekdays[iWeekday]).toUtf8(), Qt::CaseInsensitive ) )
 			return true;
-		}
+		
+		dTracing() << "  2) compare" << sLookingDayNameUtf8 << " with " << iWeekday << " -> " << i18n(vWeekdays[iWeekday]).toUtf8();
+		if( sLookingDayNameUtf8.startsWith( i18n(vWeekdays[iWeekday]).toUtf8(), Qt::CaseInsensitive ) )
+			return true;
+
+		vWeekdayUtf8 = i18n(vWeekdays[iWeekday]).toUtf8();
+		dTracing() << "  3) compare" << vWeekdayUtf8 << " with " << sLookingDayNameUtf8;
+		if( vWeekdayUtf8.startsWith( sLookingDayNameUtf8, Qt::CaseInsensitive ) )
+			return true;
+
+		dTracing() << "  4) compare" << sLookingDayNameUtf8 << " with " << kDate.toString("%a").toUtf8();
+		if( sLookingDayNameUtf8.startsWith( kDate.toString("%a"), Qt::CaseInsensitive ) )
+			return true;
+		
+		dTracing() << "  5) compare" << sLookingDayNameUtf8 << " with " << i18n(kDate.toString("%a").toUtf8().constData());
+		if( sLookingDayNameUtf8.startsWith( i18n(kDate.toString("%a").toUtf8().constData()), Qt::CaseInsensitive ) )
+			return true;
+		
 		currDate = currDate.addDays(-1);
+		kDate = kDate.addDays(-1);
 	}
 	return false;
 }
@@ -792,8 +856,187 @@ WeatherDataProcessor::Private::initIconMap(const QString & resource)
 	return true;
 }
 
+void
+WeatherDataProcessor::Private::createPropertyStringList( const YawpDay & day, YawpWeather & weather ) const
+{
+	QMap<short, Yawp::DetailsProperty> vPropertyMap;
+
+	//--- check for properties and add them to the list ---
+	if( !weather.description().isEmpty() )
+		addPropertyItem( vPropertyMap, Yawp::WeatherDescription );
+	if( weather.temperatureRealFeelHigh() < SHRT_MAX && weather.temperatureRealFeelLow() < SHRT_MAX )
+		addPropertyItem( vPropertyMap, Yawp::RealfeelTemperature );
+	if( weather.dewpoint() < SHRT_MAX )
+		addPropertyItem( vPropertyMap, Yawp::Dewpoint );
+	if( weather.pressure() < SHRT_MAX )
+		addPropertyItem( vPropertyMap, Yawp::Pressure );
+	if( weather.uvIndex() < SHRT_MAX )
+		addPropertyItem( vPropertyMap, Yawp::UV );
+	if( weather.visibility() < SHRT_MAX )
+		addPropertyItem( vPropertyMap, Yawp::Visibility );
+	if( day.sunrise().isValid() && day.sunset().isValid() )
+		addPropertyItem( vPropertyMap, Yawp::SunriseSunset );
+
+	QFont font( KGlobalSettings::generalFont() );
+	font.setBold(false);
+	font.setPixelSize(15);
+	QTextLayout textLayout;
+	textLayout.setFont(font);
+
+	QMap<short, Yawp::DetailsProperty>::const_iterator itMap = vPropertyMap.begin();
+	QString sText;
+	QString sOffsetText(2, QChar(' '));
+	QFontMetrics fm(font);
+	qreal dOffsetWidth( (qreal)fm.width(sOffsetText) );
+
+	for( ; itMap != vPropertyMap.end(); ++itMap )
+	{
+		sText = createPropertyText(*itMap, day, weather);
+
+/**** Just for debug to test the linebreak stuff. When a text needs to be wrapped around several lines.
+      than we will indent all lines expect the first one. This way it is easy to see when a new property text
+      starts.
+		switch( qrand() % 5 )
+		{
+			case 0: break;
+			case 1: case 4:	sText.append(" A little bit more text to test the line-break."); break;
+			case 2: sText.append(" xyz abc"); break;
+			case 3: sText.append(" A short text at the end."); break;
+		}
+*/
+		sText.replace('\n', QChar::LineSeparator);
+		textLayout.setText( sText );
+		QSizeF textSize = doTextLayout( textLayout, MAX_PROPERTY_TEXT_WIDTH, dOffsetWidth);
+		if( textSize.width() > MAX_PROPERTY_TEXT_WIDTH )
+		{
+			qreal dTextLength(0.0f);
+			for( int i=0; i<textLayout.lineCount(); ++i )
+				dTextLength += (i+1 == textLayout.lineCount() ? MAX_PROPERTY_TEXT_WIDTH : textLayout.lineAt(i).naturalTextWidth());
+			QFontMetrics fm(font);
+			sText = fm.elidedText(sText, Qt::ElideRight, dTextLength);
+			textLayout.setText( sText );
+			textSize = doTextLayout( textLayout, MAX_PROPERTY_TEXT_WIDTH, dOffsetWidth );
+		}
+
+		for( int i=0; i<textLayout.lineCount(); ++i )
+		{
+			QTextLine line( textLayout.lineAt(i) );
+			int iTS = line.textStart();
+			int iTW = line.textLength();
+
+			QChar::Category category( textLayout.text().at(iTS+iTW-1).category() );
+			iTW += (category == QChar::Separator_Line || category == QChar::Separator_Space ? -1 : 0);
+//			dDebug() << line.textStart() << line.textLength() << textLayout.text().mid( iTS, iTW ) << "   " << line.naturalTextWidth();
+			if( i==0 )
+				weather.propertyTextLines().append( textLayout.text().mid( iTS, iTW ) );
+			else
+				weather.propertyTextLines().append( sOffsetText + textLayout.text().mid( iTS, iTW ) );
+		}
+
+//		dDebug() << "--------------------------------------------";
+	}
+}
+
+
 inline QString
 WeatherDataProcessor::mapConditionIcon( const QString & iconName ) const
 {
 	return d->vConditionIcons.value( iconName, "weather-none-available" );
 }
+
+QString
+WeatherDataProcessor::Private::createPropertyText( const Yawp::DetailsProperty property,
+                                                   const YawpDay & day,
+                                                   const YawpWeather & weather ) const
+{
+	switch( property )
+	{
+	case Yawp::Dewpoint:
+		return i18n("Dewpoint: %1", weather.dewpoint())	+ QChar(0xB0);
+		break;
+
+	case Yawp::Pressure:
+		{
+			QString sTmp( i18n("Pressure: %1 %2",
+				weather.pressure(),
+				Utils::GetUnitString(pressureSystem)) );
+
+			if( weather.pressureTendency() != "Unavailable" &&
+			    !weather.pressureTendency().isEmpty() )
+			{
+				sTmp.append( QString("  %1")
+					.arg( i18nc("Pressure", weather.pressureTendency().toUtf8().constData())) );
+			}
+			return sTmp;
+		}
+		break;
+
+		case Yawp::RealfeelTemperature:
+		{
+			QString sHigh(i18n("N/A"));
+			QString sLow(i18n("N/A"));
+			if( weather.temperatureRealFeelHigh() < SHRT_MAX )
+				sHigh = QString::number(weather.temperatureRealFeelHigh()) + QChar(0xB0);
+			if( weather.temperatureRealFeelLow() < SHRT_MAX )
+				sLow = QString::number(weather.temperatureRealFeelLow()) + QChar(0xB0);
+			return i18n("Real feel: %1 / %2", sHigh, sLow );
+		}
+		break;
+
+		case Yawp::SunriseSunset:
+		{
+			QString sSunrise(i18n("N/A"));
+			QString sSunset(i18n("N/A"));
+			if( day.sunrise().isValid() )
+				sSunrise = KGlobal::locale()->formatTime(day.sunrise());
+			if( day.sunset().isValid() )
+				sSunset = KGlobal::locale()->formatTime(day.sunset());
+			return i18n("Sunrise at %1, sunset at %2", sSunrise, sSunset);
+		}
+		break;
+
+		case Yawp::UV:
+			return i18n("UV Index: %1  %2",
+				weather.uvIndex(), weather.uvRating());
+			break;
+
+		case Yawp::Visibility:
+			return i18n("Visibility: %1 %2",
+				weather.visibility(),
+				Utils::GetUnitString( distanceSystem ));
+				break;
+
+		case Yawp::WeatherDescription:
+			return Utils::LocalizedWeatherString(weather.description());
+			break;
+	}
+	return QString();
+}
+
+QSizeF
+WeatherDataProcessor::Private::doTextLayout( QTextLayout & textLayout, qreal dMaxWidth, qreal dOffset ) const
+{
+	qreal dHeight(0.0);
+	qreal dWidthUsed(0.0);
+	qreal dMinWidth( 0.25f * dMaxWidth );
+
+	textLayout.beginLayout();
+	while (true)
+	{
+		QTextLine line = textLayout.createLine();
+		if (!line.isValid())
+			break;
+
+//		dDebug() << "TextLayout: " << textLayout.lineCount();
+		line.setLineWidth(dMaxWidth - (textLayout.lineCount()<=1 ? 0.0f : dOffset ));
+		line.setPosition(QPointF(0, dHeight));
+		dHeight += line.height();
+		dWidthUsed = qMax(dWidthUsed, line.naturalTextWidth());
+
+		if( line.naturalTextWidth() > dMaxWidth || line.naturalTextWidth() < dMinWidth )
+			break;
+	}
+	textLayout.endLayout();
+
+	return QSizeF(dWidthUsed, dHeight);
+}
