@@ -15,7 +15,7 @@
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA            *
 \*************************************************************************/
 
 //--- LOCAL ---
@@ -28,11 +28,9 @@
 #include <QMutex>
 #include <QMutexLocker>
 
-#include <QFile>
-
-
 //--- KDE4 ---
-
+#include <Plasma/DataContainer>
+#include <KDateTime>
 
 #define TAG_TEMP		1
 #define	TAG_SPEED		2
@@ -40,11 +38,15 @@
 const QString AccuWeatherIon::IonName("accuweather");
 const QString AccuWeatherIon::ActionValidate("validate");
 const QString AccuWeatherIon::ActionWeather("weather");
+static const char vWeekdays [7][4] = {"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};
+
 
 struct AccuWeatherIon::Private
 {
 	QMap<QString, ConditionIcons>     vConditionList;
 
+	QHash<QString, KJob *>            vActiveJobs;
+	
 	//--- Search jobs ---
 	QHash<KJob *, XmlJobData *>       vSearchJobs;
 
@@ -126,34 +128,66 @@ AccuWeatherIon::updateIonSource( const QString & source )
 	// We expect the applet to send the source in the following tokenization:
 	// ionname|validate|place_name - Triggers validation of place
 	// ionname|weather|place_name|extra - Triggers receiving weather of place
-
+	dStartFunct();
+	
 	QStringList vTokens = source.split('|');
 
 	if( vTokens.size() < 3 )
 	{
         setData(source, "validate", QString("%1|timeout").arg(IonName));
+		dEndFunct();
         return true;
     }
 
-	if( vTokens.at(1) == ActionValidate )
+	if( vTokens.at(1) == ActionValidate )	// Look for places to match
 	{
-		// Look for places to match
-		findPlace( vTokens.at(2).simplified(), source );
+		QString sPlace( vTokens.at(2).simplified() );
+		if( !d->vActiveJobs.contains( QString("%1|%2").arg(sPlace).arg(ActionValidate) ) )
+			findPlace( sPlace, source );
+		dEndFunct();
 		return true;
 	}
-	else if( vTokens.at(1) == ActionWeather )
+	else if( vTokens.at(1) == ActionWeather )	// Weather data request for a specific locaion
 	{
 		if( vTokens.size() >= 4 )
 		{
-			getWeatherXmlData( vTokens.at(2).simplified(), vTokens.at(3).simplified().replace('.', '|'), source );
-			return true;
+			dDebug() << vTokens.at(3);
+			QString sPlace( vTokens.at(2).simplified() );
+			QString sLocationCode( vTokens.at(3).simplified().replace('.', '|') );
+			if( !d->vActiveJobs.contains( QString("%1|%2").arg(sLocationCode).arg(ActionWeather) ) )
+				getWeatherXmlData( sPlace, sLocationCode, source );
+			
+/*			//---- just for debug ---
+			QFile file("/tmp/ion_accuweather_Milwaukee, NC.txt");
+			if( file.open(QIODevice::ReadOnly|QIODevice::Text) )
+			{
+				struct XmlJobData * pXmlData = new XmlJobData;
+				pXmlData->sLocation = vTokens.at(2);
+				pXmlData->sSource = source;
+				
+				WeatherData * pWeather = new WeatherData;
+				pWeather->city = pXmlData->sLocation;
+				pWeather->source = pXmlData->sSource;
+				
+				pXmlData->xmlReader.addData( file.readAll() );
+				readWeatherXmlData( pXmlData->xmlReader, *pWeather );
+				updateWeatherSource( *pWeather, NULL );
+				
+				delete pXmlData;
+				delete pWeather;
+			}
+			else
+				dWarning() << "could not open file.";
+			//--- end of debug ---
+*/
 		}
 		else
-		{
-			setData( source, ActionValidate, QString("%1|invalid|single|%1").arg(IonName).arg(vTokens.at(2).simplified()) );
-			return true;
-		}
+			setData( source, ActionValidate, QString("%1|invalid|single|%2").arg(IonName).arg(vTokens.at(2).simplified()) );
+
+		dEndFunct();
+		return true;
 	}
+	dEndFunct();
 	return false;
 }
 
@@ -195,6 +229,8 @@ AccuWeatherIon::reset()
 	/* contains the same values than vImageJobs, but the key differs so that we will find it by url. */
 	d->vImageData.clear();
 	d->vImageJobs.clear();
+	
+	d->vActiveJobs.clear();
 
 #if KDE_VERSION_MINOR >= 3
 	/**  Triggered when we get initial setup data for ions that provide a list of places */
@@ -212,8 +248,10 @@ AccuWeatherIon::reset()
 void
 AccuWeatherIon::findPlace( const QString & location, const QString & source )
 {
-	KUrl url( QByteArray("http://ruan.accu-weather.com/widget/ruan/city-find.asp?location=")
-		+ QUrl::toPercentEncoding(location) );
+	dStartFunct();
+	QUrl url("http://ruan.accu-weather.com/widget/ruan/city-find.asp",  QUrl::StrictMode);
+	url.addEncodedQueryItem("location", QUrl::toPercentEncoding(location) );
+
 	KIO::TransferJob * pJob = KIO::get( url, KIO::Reload, KIO::HideProgressInfo );
 	if( pJob )
 	{
@@ -221,18 +259,20 @@ AccuWeatherIon::findPlace( const QString & location, const QString & source )
 		 *   or in d->vObservationJobs to find this certain job.
 		 */
 		pJob->setObjectName( ActionValidate );
-		pJob->addMetaData("cookies", "none"); // Disable displaying cookies
+//		pJob->addMetaData("cookies", "none"); // Disable displaying cookies
 
 		struct XmlJobData * pXmlData = new XmlJobData;
 		pXmlData->sLocation = location;
 		pXmlData->sSource = source;
 
 		d->vSearchJobs.insert( pJob, pXmlData );
+		d->vActiveJobs.insert( QString("%1|%2").arg(location).arg(ActionValidate), pJob );
 
 		connect( pJob, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
 			SLOT(setup_slotDataArrived(KIO::Job *, const QByteArray &)) );
 		connect( pJob, SIGNAL(result(KJob *)), this, SLOT(setup_slotJobFinished(KJob *)));
 	}
+	dEndFunct();
 }
 
 void
@@ -240,7 +280,9 @@ AccuWeatherIon::setup_slotDataArrived( KIO::Job * job, const QByteArray & data )
 {
 	if( data.isEmpty() || !d->vSearchJobs.contains(job) )
 		return;
+//	dStartFunct();
 	d->vSearchJobs[job]->xmlReader.addData(data);
+//	dEndFunct();
 }
 
 void
@@ -262,6 +304,7 @@ AccuWeatherIon::setup_slotJobFinished( KJob * job )
 		readSearchXmlData( pXmlData->sLocation, pXmlData->sSource, pXmlData->xmlReader );
 	}
 	d->vSearchJobs.remove( job );
+	d->vActiveJobs.remove( QString("%1|%2").arg(pXmlData->sLocation).arg(ActionValidate) );
 	job->deleteLater();
 	delete pXmlData;
 
@@ -287,7 +330,7 @@ AccuWeatherIon::readSearchXmlData( const QString & location, const QString & sou
 			iState -= 1;
 	}
 	if( xml.hasError() )
-		xml.errorString();
+		dWarning() << xml.errorString();
 	dEndFunct();
 	return !xml.hasError();
 }
@@ -303,7 +346,7 @@ AccuWeatherIon::parseSearchLocations( const QString & location, const QString &
 
 	uint iCounter = 0;
 	QString sLocations;
-
+	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -320,15 +363,17 @@ AccuWeatherIon::parseSearchLocations( const QString & location, const QString &
 		else if( xml.isStartElement() && xml.name() == "location" )
 		{
 			QXmlStreamAttributes attributes = xml.attributes();
+			QString sCity( QUrl::fromPercentEncoding(attributes.value("city").toString().toUtf8()) );
+			QString sState( QUrl::fromPercentEncoding(attributes.value("state").toString().toUtf8()) );
+			QString sLocation( QUrl::fromPercentEncoding(attributes.value("location").toString().toUtf8()) );
+
 			if( iCounter > 0 )
 				sLocations += "|";
-			sLocations += QString("place|%1, %2|extra|%3")
-				.arg( attributes.value("city").toString() )
-				.arg( attributes.value("state").toString() )
-				.arg( attributes.value("location").toString().replace('|', '.') );
+			sLocations += QString("place|%1, %2|extra|%3").arg(sCity).arg(sState).arg(sLocation.replace('|','.'));
 			iCounter += 1;
 		}
 	}
+	dEndFunct();
 }
 
 
@@ -340,33 +385,38 @@ AccuWeatherIon::parseSearchLocations( const QString & location, const QString &
 void
 AccuWeatherIon::getWeatherXmlData( const QString & location, const QString & locationCode, const QString & source )
 {
-	KUrl url( QByteArray("http://ruan.accu-weather.com/widget/ruan/weather-data.asp?location=")
-		+ QUrl::toPercentEncoding(locationCode) );
-//	dDebug() << "location =" << location << "  locationCode =" << locationCode << "  Url =" << url;
+	dStartFunct() << locationCode;
+	QUrl urlWeather("http://ruan.accu-weather.com/widget/ruan/weather-data.asp", QUrl::StrictMode);
+	urlWeather.addEncodedQueryItem("location", QUrl::toPercentEncoding(locationCode.toUtf8(), "+"));
 
-	KIO::TransferJob * pJob = KIO::get( url, KIO::Reload, KIO::HideProgressInfo );
+	dDebug() << urlWeather;
+
+	KIO::TransferJob * pJob = KIO::get( urlWeather, KIO::Reload, KIO::HideProgressInfo );
 	if( pJob )
 	{
 		/*   we are using the name to find the corresponding weatherdata for this job
 		 */
 		pJob->setObjectName( locationCode );
-		pJob->addMetaData("cookies", "none"); // Disable displaying cookies
+//		pJob->addMetaData("cookies", "none"); // Disable displaying cookies
 
 		struct XmlJobData * pXmlData = new XmlJobData;
-		d->vObservationJobs.insert( pJob, pXmlData);
 		pXmlData->sSource = source;
 		pXmlData->sLocationCode = locationCode;
 		pXmlData->sLocation = location;
 		pXmlData->imageUrl = getImageUrl( locationCode );
 
+		d->vObservationJobs.insert( pJob, pXmlData);
+		d->vActiveJobs.insert( QString("%1|%2").arg(locationCode).arg(ActionWeather), pJob );
+		
 		connectWithImageData( pXmlData->imageUrl );
 
 		connect( pJob, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
 			SLOT(slotDataArrived(KIO::Job *, const QByteArray &)) );
 		connect( pJob, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
 
-//		dDebug() << "Observation Reader: " << d->vObservationJobs.count();
+		dDebug() << "Observation Reader: " << d->vObservationJobs.count();
 	}
+	dEndFunct();
 }
 
 void
@@ -374,18 +424,30 @@ AccuWeatherIon::slotDataArrived( KIO::Job * pJob, const QByteArray & data )
 {
 	if( data.isEmpty() || !d->vObservationJobs.contains(pJob) )
 		return;
-	d->vObservationJobs[pJob]->xmlReader.addData(data);
+//	dStartFunct();
+	XmlJobData * pJobData = d->vObservationJobs[pJob];
+	pJobData->xmlReader.addData(data);
+/*	
+	QFile file(QString("/tmp/ion_accuweather_%1.txt").arg(pJobData->sLocation.replace("|","_")));
+	if( file.open(QIODevice::Append|QIODevice::Text) )
+	{
+		file.write(data);
+		file.close();
+	}
+*/
+//	dEndFunct();
 }
 
 void
 AccuWeatherIon::slotJobFinished( KJob * pJob )
 {
+	dStartFunct();
 	if( !d->vObservationJobs.contains(pJob) )
 	{
 		dWarning() << "Does not contain current xml-job!!";
+		dEndFunct();
 		return;
 	}
-	dStartFunct();
 	XmlJobData * pXmlData = d->vObservationJobs[pJob];
 	ImageData  * pImageData = NULL;
 
@@ -409,6 +471,9 @@ AccuWeatherIon::slotJobFinished( KJob * pJob )
 		WeatherData * pWeather = new WeatherData;
 		pWeather->city = pXmlData->sLocation;
 		pWeather->source = pXmlData->sSource;
+		pWeather->iUtcHourOffset = 0;
+		pWeather->iUtcMinutesOffset = 0;
+
 		readWeatherXmlData( pXmlData->xmlReader, *pWeather );
 
 		if( pImageData )
@@ -442,6 +507,7 @@ AccuWeatherIon::slotJobFinished( KJob * pJob )
 	}
 
 	d->vObservationJobs.remove( pJob );
+	d->vActiveJobs.remove( QString("%1|%2").arg(pXmlData->sLocationCode).arg(ActionWeather) );
 	pJob->deleteLater();
 	delete pXmlData;
 
@@ -459,17 +525,22 @@ AccuWeatherIon::readWeatherXmlData( QXmlStreamReader & xml, WeatherData & weathe
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
-//		dDebug() << xml.name() << iState;
+//		dTracing() << xml.name() << iState;
 		if( xml.isStartElement() )
 		{
 			if( iState == 0 && xml.name().compare("adc_database") == 0 )
 				iState = 1;
-			else if( iState == 1 && xml.name().compare("units") == 0 )
-				readUnits( xml, weather );
-			else if( iState == 1 && xml.name().compare("currentconditions") == 0 )
-				readCurrentConditions( xml, weather );
-			else if( iState == 1 && xml.name().compare("forecast") == 0 )
-				iState = 2;
+			else if( iState == 1 )
+			{
+				if( xml.name().compare("units") == 0 )
+					readUnits( xml, weather );
+				else if( xml.name().compare("local") == 0 )
+					readLocal( xml, weather );
+				else if( xml.name().compare("currentconditions") == 0 )
+					readCurrentConditions( xml, weather );
+				else if( xml.name().compare("forecast") == 0 )
+					iState = 2;
+			}
 			else if( iState == 2 && xml.name().compare("day") == 0 )
 			{
 				bool bOk = false;
@@ -501,9 +572,9 @@ AccuWeatherIon::readUnits( QXmlStreamReader & xml, WeatherData & weather )
 {
 	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("units") == 0,
 	            "AccuWeather Ion",
-	            "AccuWeatherIon::readUnits - current tag is wrong!" );
+	            "AccuWeatherIon::readUnits - wrong tag to parse!" );
 
-//	dStartFunct();
+	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -513,12 +584,9 @@ AccuWeatherIon::readUnits( QXmlStreamReader & xml, WeatherData & weather )
 		{
 //			dDebug() << xml.name().toString();
 			if( xml.name().compare("temp") == 0 )
-				weather.iTempSystem = ( xml.readElementText().compare("F") == 0 ?
-					FAHRENHEIT : CELSIUS );
+				weather.iTempSystem = ( xml.readElementText().compare("F") == 0 ? FAHRENHEIT : CELSIUS );
 			else if( xml.name().compare("speed") == 0 )
-				weather.iSpeedSystem = ( xml.readElementText().compare("MPH") == 0 ?
-					MPH : KPH
-					);
+				weather.iSpeedSystem = ( xml.readElementText().compare("MPH") == 0 ? MPH : KPH );
 			else if( xml.name().compare("dist") == 0 )
 			{
 				QString sDist = xml.readElementText();
@@ -542,7 +610,45 @@ AccuWeatherIon::readUnits( QXmlStreamReader & xml, WeatherData & weather )
 	}
 	if( xml.hasError() )
 		dWarning() << xml.errorString();
-//	dEndFunct();
+	dEndFunct();
+}
+
+void
+AccuWeatherIon::readLocal( QXmlStreamReader & xml, WeatherData & weather )
+{
+	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("local") == 0,
+	            "AccuWeather Ion",
+	            "AccuWeatherIon::readLocal - wrong tag to parse!" );
+	dStartFunct();
+	while( !xml.atEnd() )
+	{
+		xml.readNext();
+		if( xml.isEndElement() && xml.name().compare("local") == 0 )
+			break;
+		else if( xml.isStartElement() )
+		{
+			if( xml.name().compare("lat") == 0 )
+				weather.sLatitude = xml.readElementText();
+			else if( xml.name().compare("lon") == 0 )
+				weather.sLongitude = xml.readElementText();
+			else if( xml.name().compare("time") == 0 )
+				weather.sObservationPeriode = xml.readElementText();
+			else if( xml.name().compare("timeZone") == 0 )
+			{
+				QString sTimeZone(xml.readElementText());
+				int iPos( sTimeZone.indexOf(QChar(':')) );
+				if( iPos > 0 )
+				{
+					weather.iUtcHourOffset = sTimeZone.left(iPos).toShort();
+					weather.iUtcMinutesOffset = sTimeZone.right(sTimeZone.length()-iPos-1).toShort();
+				}
+			}
+		}
+	}
+	if( xml.hasError() )
+		dWarning() << xml.errorString();
+	dEndFunct();
+
 }
 
 void
@@ -550,9 +656,9 @@ AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & wea
 {
 	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("currentconditions") == 0,
 	            "AccuWeather Ion",
-	            "AccuWeatherIon::readCurrentConditions - current tag is wrong!" );
-
-//	dStartFunct();
+	            "AccuWeatherIon::readCurrentConditions - wrong tag to parse!" );
+	
+	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -600,7 +706,7 @@ AccuWeatherIon::readCurrentConditions( QXmlStreamReader & xml, WeatherData & wea
 	}
 	if( xml.hasError() )
 		dWarning() << xml.errorString();
-//	dEndFunct();
+	dEndFunct();
 }
 
 void
@@ -609,7 +715,7 @@ AccuWeatherIon::readForecastConditions( QXmlStreamReader & xml, ForecastDay & fo
 	Q_ASSERT_X( xml.isStartElement() && xml.name().compare("day") == 0,
 	            "AccuWeather Ion",
 	            "AccuWeatherIon::readForecastConditions - current tag is wrong!" );
-//	dStartFunct();
+	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -625,14 +731,23 @@ AccuWeatherIon::readForecastConditions( QXmlStreamReader & xml, ForecastDay & fo
 			else if( xml.name().compare("sunset") == 0 )
 				forecast.sunset = xml.readElementText();
 			else if( xml.name().compare("daytime") == 0 )
+			{
+				dTracing() << "Parse day time values:";
 				readWeatherConditions( xml, forecast.DayTime );
+			}
 			else if( xml.name().compare("nighttime") == 0 )
+			{
+				dTracing() << "Parse night time values:";
 				readWeatherConditions( xml, forecast.NightTime );
+			}
 		}
 	}
+	dTracing() << "Obsdate:" << forecast.obsdate;
+	dTracing() << "Sunrise/Sunset:" << forecast.sunrise << " / " << forecast.sunset;
+
 	if( xml.hasError() )
 		dWarning() << xml.errorString();
-//	dEndFunct();
+	dEndFunct();
 }
 
 void
@@ -643,7 +758,7 @@ AccuWeatherIon::readWeatherConditions( QXmlStreamReader & xml, ForecastCondition
 		"AccuWeather Ion",
 		"AccuWeatherIon::readForecastConditions - current tag is wrong!" );
 
-//	dStartFunct();
+	dStartFunct();
 	while( !xml.atEnd() )
 	{
 		xml.readNext();
@@ -674,15 +789,23 @@ AccuWeatherIon::readWeatherConditions( QXmlStreamReader & xml, ForecastCondition
 				conditions.uvIndex = xml.readElementText();
 		}
 	}
+/*	dTracing() << "Text:" << conditions.weathertext;
+	dTracing() << "Icon:" << conditions.weathericon;
+	dTracing() << "Temp:" << conditions.hightemperature << "/" << conditions.lowtemperature;
+	dTracing() << "Realfeel:" << conditions.realfeelhigh << "/" << conditions.realfeellow;
+	dTracing() << "Wind:" << conditions.windspeed << conditions.winddirection;
+	dTracing() << "WindGust:" << conditions.windgust;
+	dTracing() << "Max UV:" << conditions.uvIndex;
+*/
 	if( xml.hasError() )
 		dWarning() << xml.errorString();
-//	dEndFunct();
+	dEndFunct();
 }
 
 void
 AccuWeatherIon::updateWeatherSource( const WeatherData & weather, const ImageData * pImageData )
 {
-//	dStartFunct()	<< weather.source;
+	dStartFunct()	<< weather.source;
 	removeAllData( weather.source );	// clear the old values
 	setData(weather.source, Data());	// start the update timer
 
@@ -693,8 +816,8 @@ AccuWeatherIon::updateWeatherSource( const WeatherData & weather, const ImageDat
 	setData( weather.source, "Visibility Unit", QString::number(weather.iDistanceSystem) );
 
 	setData( weather.source, "Place", weather.city );
-	setData( weather.source, "Current Conditions", stringConverter(weather.weathertext) );
 	setData( weather.source, "Condition Icon", stringConverter(weather.weathericon) );
+	setData( weather.source, "Current Conditions", stringConverter(weather.weathertext) );
 	setData( weather.source, "Dewpoint", stringConverter(weather.dewpoint) );
 	setData( weather.source, "Humidity", stringConverter(weather.humidity) );
 	setData( weather.source, "Pressure", stringConverter(weather.pressure) );
@@ -711,40 +834,75 @@ AccuWeatherIon::updateWeatherSource( const WeatherData & weather, const ImageDat
 	setData( weather.source, "Wind Speed", stringConverter(weather.windspeed) );
 	setData( weather.source, "Wind Gust", stringConverter(weather.windgusts) );
 	setData( weather.source, "Windchill", stringConverter(weather.windchill) );
+	
+	if( !weather.sLatitude.isEmpty() && !weather.sLongitude.isEmpty() )
+	{
+		setData( weather.source, "Longitude", weather.sLongitude );
+		setData( weather.source, "Latitude", weather.sLatitude );
+	}
 
-	QDate currDate;
 	if( weather.vForecasts.count() > 0 )
 	{
-		currDate = QDate::fromString(weather.vForecasts.at(0)->obsdate, "M/d/yyyy");
+		QDate currDate( QDate::fromString(weather.vForecasts.at(0)->obsdate, "M/d/yyyy") );
+		dInfo() << "Observation day: " << currDate;
 
 		QTime sunrise( QTime::fromString(weather.vForecasts.at(0)->sunrise, "h:mm AP") );
 		QTime sunset( QTime::fromString(weather.vForecasts.at(0)->sunset, "h:mm AP") );
-		if( sunrise.isValid() )
+		if( sunrise.isValid() && sunset.isValid() )
+		{
+#if KDE_VERSION_MINOR <= 2
 			setData( weather.source, "Sunrise At", sunrise.toString("h:m") );
-		if( sunset.isValid() )
 			setData( weather.source, "Sunset At", sunset.toString("h:m") );
+#else
+			KDateTime dateTime( currDate, sunrise, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
+			if( dateTime.isValid() )
+				setData( weather.source, "Sunrise At", dateTime.toString("%A %B %e, %Y at %H:%M %Z") );
+		
+			dateTime = KDateTime( currDate, sunset, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
+			if( dateTime.isValid() )
+				setData( weather.source, "Sunset At", dateTime.toString("%A %B %e, %Y at %H:%M %Z") );
+#endif
+		}
+
+		QTime observationPeriode( QTime::fromString(weather.sObservationPeriode, "h:mm") );
+		if( observationPeriode.isValid() )
+		{
+			KDateTime dateTime( currDate, observationPeriode, KDateTime::Spec::OffsetFromUTC(weather.iUtcHourOffset) );
+			if( dateTime.isValid() )
+				setData( weather.source, "Observation Period", dateTime.toString("%d.%m.%Y @ %H:%M") );
+		}
+
+		short iDay(0);
+		QList<ForecastDay *>::const_iterator itDay = weather.vForecasts.constBegin();
+		for( ; itDay != weather.vForecasts.constEnd(); ++itDay, ++iDay )
+		{
+			QString sDayName("???");
+			int iWeekday = currDate.dayOfWeek();
+			if( iWeekday >= 1 && iWeekday<=7 )
+				sDayName = i18n(vWeekdays[iWeekday-1]);
+
+			updateWeatherCondition( weather.source, iDay, sDayName, true, (*itDay)->DayTime );
+			updateWeatherCondition( weather.source, iDay, sDayName, false, (*itDay)->NightTime );
+
+			if( iDay > 0 )
+				updateSun( weather.source, iDay, sDayName, **itDay );
+
+			currDate = currDate.addDays(1);
+		}
+		setData( weather.source, "Total Weather Days", QString::number(2*weather.vForecasts.count()) );
 	}
-	for( int iDay = 0; iDay < weather.vForecasts.count(); ++iDay )
-	{
-		QString sDayName( currDate.toString("ddd") );
-		updateWeatherCondition( weather.source, iDay, sDayName, true, weather.vForecasts.at(iDay)->DayTime );
-		updateWeatherCondition( weather.source, iDay, sDayName, false, weather.vForecasts.at(iDay)->NightTime );
-
-		if( iDay > 0 )
-			updateSun( weather.source, iDay, sDayName, *weather.vForecasts.at(iDay) );
-
-		currDate = currDate.addDays(1);
-	}
-
-	setData( weather.source, "Total Weather Days", QString::number(2*weather.vForecasts.count()) );
 
 	if( pImageData && !pImageData->image.isNull() )
 		setData( weather.source, "Satellite Map", pImageData->image );
 
 	setData( weather.source, "Credit", i18n("Supported by AccuWeather") );
 	setData( weather.source, "Credit Url", "http://www.accuweather.com" );
-
-//	dEndFunct();
+	
+/*	Plasma::DataContainer * pCon = containerForSource(weather.source);
+	if( pCon )
+		dTracing() << pCon->data();
+*/
+	dEndFunct();
 }
 
 void
@@ -842,9 +1000,11 @@ AccuWeatherIon::getImageUrl( const QString & sLocationCode ) const
 		{
 			if (country == QLatin1String("IN") )
 				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isindia.jpg");
+			else if( country == QLatin1String("RU") )
+				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/iseurm.jpg");
 			else
 				sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/isasia.jpg");
-                }
+		}
 		else if( area == QLatin1String("MEA") )
 			sUrl = QByteArray("http://sirocco.accuweather.com/sat_mosaic_400x300_public/IR/ismide.jpg");
 		else if( area == QLatin1String("AFR") )
@@ -866,13 +1026,14 @@ AccuWeatherIon::getImageUrl( const QString & sLocationCode ) const
 void
 AccuWeatherIon::connectWithImageData( const QByteArray & imageUrl )
 {
+	dStartFunct();
 	ImageData * pImageData = NULL;
 	if( !d->vImageData.contains( imageUrl ) )
 	{
 		KIO::TransferJob * pJob = KIO::get( KUrl(imageUrl), KIO::Reload, KIO::HideProgressInfo );
 		if( pJob )
 		{
-			pJob->addMetaData("cookies", "none"); // Disable displaying cookies
+//			pJob->addMetaData("cookies", "none"); // Disable displaying cookies
 			pImageData = new ImageData;
 			pImageData->imageUrl = imageUrl;
 			pImageData->bReady = false;
@@ -890,6 +1051,7 @@ AccuWeatherIon::connectWithImageData( const QByteArray & imageUrl )
 		pImageData = d->vImageData[imageUrl];
 		pImageData->iLocationCounter += 1;
 	}
+	dEndFunct();
 }
 
 void
@@ -897,7 +1059,9 @@ AccuWeatherIon::image_slotDataArrived( KIO::Job * pJob, const QByteArray & data
 {
 	if( data.isEmpty() || !d->vImageJobs.contains(pJob) )
 		return;
+//	dStartFunct();
 	d->vImageJobs[pJob]->fetchedData.append( data );
+//	dEndFunct();
 }
 
 void
